﻿/*
*  drmTEST plugin for Movian Media Center
*
*  Copyright (C) 2023-2024 dean
*/

var PREFIX		= 'drmTEST';
var manifest 	= JSON.parse(Plugin.manifest);
var logo		= Plugin.path + "logo.png";
var http		= require('movian/http');
var prop		= require('movian/prop');
var popup		= require('native/popup');

var movian_page	= require('movian/page');
var movian_service  = require('movian/service');

var DRM_NOT_SUPPORTED = -1;
var DRM_NOT_INSTALLED = 0;
var DRM_INSTALLED = 1;

var subtitle_format = "default";
var movianDRM 		= DRM_NOT_SUPPORTED;
var movianDRM_ver	= "0.0.0";

prop.subscribeValue(prop.global.androidTV.subtitle_format,	function(value) { if(typeof value === 'string') { subtitle_format = value; } } );
prop.subscribeValue(prop.global.androidTV.movianDRM, 		function(value) { if(typeof value === 'number') { movianDRM = value; } } );
prop.subscribeValue(prop.global.androidTV.movianDRM_ver,	function(value) { if(typeof value === 'string') { movianDRM_ver = value; } } );

var service = movian_service.create(manifest.title, PREFIX + ":start", "video", true, logo);

/*
	headers: array object to push custom HTTP headers to
	scope:	all - headers added to all HTTP requests
			drm - headers added only to license requests
			clr - headers added only to non-drm requests
	name:	header name
	value:	header value
	
*/
function add_header(headers, scope, name, value)
{
	if(!name || !value) return headers;
	
	var suffix;
	
	if(scope == "drm" || scope == "clr") 
		suffix = "_" + scope;
	else 
		suffix = "";
	
	var tmp = {};
	tmp["hdr" + suffix] = name;
	tmp["val" + suffix] = value;
	
	headers.push(tmp);
	return headers;
}

new movian_page.Route(PREFIX + ":start", function(page) 
{
	page.type = "directory";
	page.contents = "items";
	page.metadata.logo = logo;
	page.metadata.icon = logo;
	page.metadata.title = manifest.title;

	var samples			= [];
	
	if(movianDRM == DRM_NOT_INSTALLED)
	{
		popup.message('<font size="4" color="#FF1010">movianDRM is not installed</font><br><hr><br><font size="3" color="#C0C0C0">Click [OK] to open Movian DRM install page (Settings/General/DRM Upgrade)<br><hr></font>', true);
		page.redirect("showtime:upgradedrm");
		return;
	}
	else if(movianDRM == DRM_NOT_SUPPORTED)
	{
		popup.message('<font size="4" color="#FF1010">movianDRM is not supported on your platform</font><br><hr><br><font size="3" color="#C0C0C0">Android 5.1 and Movian 5.0.752 or later versions required<br><hr></font>', true);
		return;
	}
	
	console.log("Current DRM version: "+movianDRM_ver);

	/*
	#
	# more info: https://movian.deanbg.com/Android/movianDRM/how_to_use_in_plugins.txt
	# sample full example
	#
	samples.push({
		uri: 			'https://address_of_the_video_manifest',
		canonical_url:	'drmTEST:video:video_id', // setting this will allow proper playback resume/mark as seen
		name:			encodeURIComponent(title), 
		sub_title:		encodeURIComponent(description.substr(0, 512)), 
		artwork_uri:	encodeURIComponent('https://address_of_the_icon'),
		extension:		'mpd', 					// mpd or m3u8
		mime_type:		'application/dash+xml', // application/dash+xml or application/x-mpegURL
		drm_scheme:		'widevine', 			// DRM scheme if protected (widevine, playready, clearkey)
		drm_license_uri: 'https://drm_license_uri',
	});
	*/

	// CLEAR (NO-DRM) SAMPLES

	samples.push({
		uri: "https://storage.googleapis.com/exoplayer-test-media-1/60fps/bbb-clear-1080/manifest.mpd", 
		canonical_url: "drmTEST:sample_1",
		name: encodeURIComponent("Clear DASH - 1080p@60fps"),
		extension: "mpd",
		mime_type: "application/dash+xml",
		sub_title: "Big Buck Bunny"
	});

	samples.push({
		uri: "https://devstreaming-cdn.apple.com/videos/streaming/examples/img_bipbop_adv_example_ts/master.m3u8", 
		canonical_url: "drmTEST:sample_2",
		name: encodeURIComponent("Clear HLS - HD (fMP4)"),
		extension: "m3u8",
		mime_type: "application/x-mpegURL",
		sub_title: "Apple HLS Multi-stream Test"
	});
	
	samples.push({
		uri: "https://storage.googleapis.com/wvmedia/clear/h264/tears/tears.mpd", 
		name: encodeURIComponent("Clear DASH - HD (MP4, H264)"), 
		sub_title: "movianDRM Test Sample"
	});

	samples.push({
		uri: "https://storage.googleapis.com/wvmedia/clear/hevc/tears/tears.mpd", 
		name: encodeURIComponent("Clear DASH - HD (MP4, H265)"), 
		sub_title: "movianDRM Test Sample"
	});

	samples.push({
		uri: "https://storage.googleapis.com/wvmedia/clear/vp9/tears/tears.mpd", 
		name: encodeURIComponent("Clear DASH - HD (WebM, VP9)"), 
		sub_title: "movianDRM Test Sample"
	});


	// DRM SAMPLES
	
	samples.push({
		uri: "https://storage.googleapis.com/wvmedia/cenc/h264/tears/tears.mpd", 
		drm_scheme: "widevine",
        drm_license_uri: "https://proxy.uat.widevine.com/proxy?video_id=2015_tears&provider=widevine_test",
		name: encodeURIComponent("Widevine DASH (MP4, H264) HD (cenc)"), 
		sub_title: "movianDRM Test Sample"
	});

	samples.push({
		uri: "https://storage.googleapis.com/wvmedia/cenc/vp9/tears/tears.mpd", 
		drm_scheme: "widevine",
        drm_license_uri: "https://proxy.uat.widevine.com/proxy?video_id=2015_tears&provider=widevine_test",
		name: encodeURIComponent("Widevine DASH (WebM, VP9) HD (cenc, full-sample)"), 
		sub_title: "movianDRM Test Sample"
	});

	samples.push({
		uri: "https://storage.googleapis.com/wvmedia/cenc/hevc/tears/tears.mpd", 
		drm_scheme: "widevine",
        drm_license_uri: "https://proxy.uat.widevine.com/proxy?video_id=2015_tears&provider=widevine_test",
		name: encodeURIComponent("Widevine DASH (MP4, H265) HD (cenc)"), 
		sub_title: "movianDRM Test Sample"
	});


	var mDRM	= [];
	var headers = [];
	
	headers = add_header(headers, "all", "User-Agent", "drmTEST/1.0.0");
	/*
	headers = add_header(headers, "drm", "X-Custom-DRM", "DrmKeyHeader");
	headers = add_header(headers, "clr", "X-Custom-CLR", "ClearContentHeader");
	headers = add_header(headers, "all", "X-Custom-ALL", "AllContentHeader");
	headers = add_header(headers, "all", "Cookie", "drmTEST=1");
	*/
	mDRM.push({
		headers:				JSON.stringify(headers),
		subtitle_formatting:	subtitle_format,
	});	

	for(var i = 0; i < samples.length; i++)
			mDRM.push(samples[i]);

	var drm_all = 'movianDRM:dash:'+encodeURIComponent('json://'+JSON.stringify(mDRM));
	
	page.appendItem(drm_all, 'directory', {	icon: logo,	title: "Play all" });

	page.appendItem("", "separator", {title: "Sample videos"});

	for(var i = 0; i < samples.length; i++)
	{
		var mDRM_single = [];

		var headers_single = [];
		
		headers_single = add_header(headers_single, "all", "User-Agent", "drmTEST/1.0.0");
		/*
		headers_single = add_header(headers_single, "drm", "X-Custom-DRM", "DrmKeyHeader");
		headers_single = add_header(headers_single, "clr", "X-Custom-CLR", "ClearContentHeader");
		headers_single = add_header(headers_single, "all", "X-Custom-ALL", "AllContentHeader");
		headers_single = add_header(headers_single, "all", "Cookie", "drmTEST=1");
		*/
		mDRM_single.push({
			headers:				JSON.stringify(headers_single),
			subtitle_formatting:	subtitle_format,
		});	
		
		mDRM_single.push(samples[i]);
		
		var drm_link = 'movianDRM:dash:'+encodeURIComponent('json://'+JSON.stringify(mDRM_single));
		
		page.appendItem(drm_link, 'directory', { icon: logo, title: decodeURIComponent(samples[i].name) });
	}
	
	// Auto-start playback of all samples (uncomment next line)
	// page.openurl(drm_all);
});
